import time

import pyperclip
import selenium
from selenium import webdriver
from selenium.common import StaleElementReferenceException, JavascriptException

from auto_loc.locators import get_xpath, get_css_selector
from auto_loc.utils import highlight_element, save_locators_to_json
from pynput import keyboard   # Для отслеживания нажатий клавиш


class WebInspector:
    def __init__(self, browser="chrome", headless=False, test_attributes: list=None):
        """Инициализация инспектора."""
        self.test_attributes = test_attributes
        self.options = webdriver.ChromeOptions()

        if headless:
            self.options.add_argument("--headless")
        if browser == 'chrome':
            self.driver = webdriver.Chrome(options=self.options)
            # self.driver.maximize_window()
        else:
            raise ValueError("Currently only Chrome is supported")

        self.last_element = None  # Последний выбранный элемент
        self.exit_flag = False  # Флаг для выхода


    def start(self, url):
        """Запуск инспектора на указанной странице."""
        self.driver.get(url)
        print(f"Открыл страницу {url}")
        print("Нажмите 'q' для выхода.")
        self._inspect_page()

    def _inspect_page(self):
        """Инспектор с возможностью:
        - Одинарный клик — блокировка действия и генерация локатора.
        - Клик с Shift — стандартное взаимодействие без генерации локатора.
        - Двойной клик — стандартное взаимодействие с элементом."""

        print(
            "Инспектор запущен. "
            "Одинарный клик — блокировка действия и генерация локатора, "
            "Shift + клик — стандартное взаимодействие без генерации локатора, "
            "Двойной клик — стандартное взаимодействие.")

        # Слушаем клики на странице
        self.driver.execute_script("""
                document.addEventListener('click', function(event) {
                    console.log('Click event detected');
                    if (event.detail === 1) {  // Одинарный клик
                        console.log('Single click detected');
                        if (!event.shiftKey) {  // Одинарный клик без Shift
                            console.log('Single click without Shift detected');
                            let element = event.target;
                            window.element_path = element;  // Сохраняем элемент в глобальной переменной
                            event.preventDefault();  // Блокируем стандартное действие клика
                            event.stopPropagation();  // Останавливаем дальнейшую обработку события
                        } else if (event.shiftKey) {  // Клик с Shift
                            console.log('Click with Shift detected');
                            // Оставляем стандартное поведение без сохранения элемента
                            return;  // Ничего не делаем, стандартное поведение браузера
                        }
                    }
                }, true);

                document.addEventListener('dblclick', function(event) {
                    console.log('Double click detected');
                    // Двойной клик: Оставляем стандартное поведение
                }, true);
            """)

        # Устанавливаем обработчик для клавиши 'q'
        listener = keyboard.Listener(on_press=self._on_key_press)
        listener.start()  # Запускаем слушатель в отдельном потоке

        retry_count = 0  # Счётчик повторных попыток
        max_retries = 2  # Максимальное количество попыток

        # Основной цикл работы инспектора
        while not self.exit_flag:
            try:
                element = self.driver.execute_script("return window.element_path;")

                # Проверяем, есть ли новый клик и отличается ли он от предыдущего элемента
                if element and element != self.last_element:
                    self.last_element = element
                    self._highlight_and_generate_locator(element, self.test_attributes)
                    retry_count = 0  # Сбрасываем счётчик при успешной операции
                    # Сбрасываем переменную в браузере, чтобы ждать новый клик
                    self.driver.execute_script("window.element_path = null;")

            except selenium.common.exceptions.StaleElementReferenceException as e:
                print(f"Stale element: {str(e)}. Попытка {retry_count + 1} из {max_retries}")
                retry_count += 1
                if retry_count >= max_retries:
                    print("Пропускаем элемент из-за устаревания.")
                    retry_count = 0
                    # Сбрасываем переменную в браузере
                    self.driver.execute_script("window.element_path = null;")
                    self.last_element = None  # Обнуляем последний элемент, чтобы искать новый

            except selenium.common.exceptions.JavascriptException as e:
                print(f"Ошибка JavaScript: {str(e)}")
                retry_count += 1
                if retry_count >= max_retries:
                    print("Пропускаем элемент из-за ошибки JavaScript.")
                    retry_count = 0
                    # Сбрасываем переменную в браузере
                    self.driver.execute_script("window.element_path = null;")
                    self.last_element = None

            except Exception as e:
                print(f"Общая ошибка: {str(e)}")
                retry_count += 1
                if retry_count >= max_retries:
                    print("Пропускаем элемент из-за общей ошибки.")
                    retry_count = 0
                    # Сбрасываем переменную в браузере
                    self.driver.execute_script("window.element_path = null;")
                    self.last_element = None

            time.sleep(0.5)  # Ждем перед следующей проверкой

        print("Выход из инспектора...")
        listener.stop()  # Останавливаем слушатель после выхода


    def _highlight_and_generate_locator(self, element, test_attributes):
        """Подсвечивание и генерация локатора с обработкой StaleElementReferenceException."""
        try:
            # Переобновляем элемент, если он стал устаревшим
            if not self._is_element_stale(element):
                # Подсвечиваем элемент
                highlight_element(self.driver, element)

                # Генерируем оба локатора
                xpath = get_xpath(self.driver, element, test_attributes)
                css_selector = get_css_selector(self.driver, element, test_attributes)

                # Выводим локаторы
                print(f"\nXPATH локатор: {xpath}")
                print(f"CSS локатор: {css_selector}")

                # Копируем XPATH в буфер обмена
                pyperclip.copy(xpath)
                print("XPATH локатор скопирован в буфер обмена!")

                # Сохраняем локаторы в файл
                save_locators_to_json(xpath, css_selector)
            else:
                print("Элемент устарел. Повторный поиск элемента.")
                # Переобновляем элемент (например, находим его снова)
                # element = self.driver.find_element(By.XPATH, "...") # пример поиска элемента заново
        except StaleElementReferenceException:
            print("Ошибка: Элемент устарел и недоступен. Попробуйте кликнуть снова.")
        except JavascriptException as e:
            print(f"Ошибка JavaScript: {e}")

    def _is_element_stale(self, element):
        """Проверка, является ли элемент устаревшим."""
        try:
            # Проверяем, доступен ли элемент
            element.is_enabled()
            return False
        except StaleElementReferenceException:
            return True

    def _on_key_press(self, key):
        """Устанавливает флаг выхода при нажатии 'q'."""
        try:
            if key.char == 'q':
                self._set_exit_flag()
                print("\nExit flag set to True.")
        except AttributeError:
            pass

    def _set_exit_flag(self):
        """Установка флага выхода при нажатии 'q'."""
        print("\nВыход из инспектора...")
        self.exit_flag = True

    def quit(self):
        """Закрытие браузера."""
        self.driver.quit()


# Пример использования
if __name__ == "__main__":
    list_attributes = ["data-test-id", "data-e2e", "test-id"]
    # url = "https://demoqa.com/automation-practice-form"
    url = "https://3d4medical.com/"
    inspector = WebInspector(test_attributes=list_attributes)
    inspector.start(url)
    inspector.quit()



import argparse
import os

def get_project_root(auto_loc_path):
    project_root = auto_loc_path
    while not auto_loc_path.endswith("venv"):
        project_root = os.path.dirname(project_root)
        if project_root.endswith("venv"):
            project_root = os.path.dirname(project_root)
            break
    return project_root

def create_setup(project_root, lib_root):
    # Read text from the source .txt file
    with open(os.path.join(lib_root, "run_text.txt"), "r") as txt_file:
        content = txt_file.read()

    # Path to the check.py file
    check_py_path = os.path.join(project_root, "web_inspector.py")

    # Check if the check.py file exists and is not empty
    if not os.path.exists(check_py_path) or os.path.getsize(check_py_path) == 0:
        # Create the .py file in the project root
        with open(check_py_path, "w") as py_file:
            py_file.write(content)

        # Add check.py to .gitignore
        gitignore_path = os.path.join(project_root, ".gitignore")
        with open(gitignore_path, "a") as gitignore_file:
            gitignore_file.write("\nweb_inspector.py\n")


def main():
    # Define the root directory of the project
    lib_root = os.path.dirname(os.path.abspath(__file__))
    project_root = get_project_root(lib_root)
    parser = argparse.ArgumentParser(description="Run Web Inspector locally")
    parser.add_argument('--run', action='store_true', help="run inspector")
    args = parser.parse_args()
    if args.run:
        create_setup(project_root, lib_root)


if __name__ == '__main__':
    # e.g.: create-setup --run
    main()



from auto_loc.inspector import WebInspector

def test_inspector(locator_attributes: list=None, url: str=None):
    inspector = WebInspector(test_attributes=locator_attributes)
    inspector.start(url)
    inspector.quit()


if __name__ == '__main__':
    url = "https://demoqa.com/automation-practice-form"
    locator_attributes = ["data-test-id", "data-e2e", "test-id"]
    test_inspector(locator_attributes, url)



"""
run via command line using default test arguments:
e.g.: run-inspector --url "https://demoqa.com/automation-practice-form"
"""


import json
import time

def highlight_element(driver, element, duration=1):
    """Подсветка выбранного элемента на странице с помощью JavaScript."""
    original_style = element.get_attribute('style')
    driver.execute_script("arguments[0].setAttribute('style', arguments[1]);", element, "border: 3px solid red;")
    time.sleep(duration)  # Подсвечиваем элемент на указанное время
    driver.execute_script("arguments[0].setAttribute('style', arguments[1]);", element, original_style)

def save_locators_to_json(xpath, css_selector, file_name="locators.json"):
    locators = {
        "xpath": xpath,
        "css_selector": css_selector
    }
    with open(file_name, "a") as file:
        json.dump(locators, file)
        file.write("\n")



import argparse
from auto_loc import WebInspector

def main():
    list_attributes = ["data-test-id", "data-e2e", "test-id"]
    parser = argparse.ArgumentParser(description="Run Web Inspector")
    parser.add_argument('--url', type=str, help="URL to inspect", required=True)
    args = parser.parse_args()

    inspector = WebInspector(test_attributes=list_attributes)
    inspector.start(args.url)
    inspector.quit()

if __name__ == "__main__":
    # e.g.: run-inspector --url "https://demoqa.com/automation-practice-form"
    main()



import re
from venv import logger

import pyperclip
from selenium.webdriver.common.by import By


def get_xpath(driver, element, test_attributes):
    """Генерация короткого XPATH для элемента с приоритетом поиска по id, тексту, name и другим атрибутам, а также выводом placeholder."""
    first_tag = element.tag_name.lower()
    if first_tag == "svg" or len(first_tag) < 3 or first_tag == "path":
        tag = "*"
    else:
        tag = first_tag

    def is_dynamic_attribute(value):
        # Проверка на динамически сгенерированные атрибуты, например id со случайными числами
        return value.isdigit() or re.match(r".*-\d+", value)

    def generate_primary_xpath():

        def add_index_if_needed(xpath, paragraph: bool = False):
            siblings = driver.find_elements(By.XPATH, xpath)
            if len(siblings) > 1:
                index = siblings.index(element) + 1
                return f"{xpath}[{index}]"
            if paragraph:
                new_path = xpath + "/p"
                elem = driver.find_elements(By.XPATH, new_path)
                if len(elem) > 1:
                    index = elem.index(element) + 1
                    return f"{new_path}[{index}]"
                return new_path
            else:
                return xpath

        # Priority 0: Search by test attributes
        primary_xpath = None
        for attr in test_attributes:
            attr_value = element.get_attribute(attr)
            if attr_value:
                return add_index_if_needed(f"//{tag}[@{attr}='{attr_value}']")

        if not primary_xpath:
            # Priority 1: Search by id
            element_id = element.get_attribute("id")
            if element_id and not is_dynamic_attribute(element_id):
                return add_index_if_needed(f"//{tag}[@id='{element_id}']")
            else:
                # Priority 2: Search by text
                text = element.text.strip()
                check_text = ["title", "lower", "upper", "capitalize"]
                if text and len(text) < 20:
                    for check in check_text:
                        text_new = getattr(text, check)()
                        try:
                            if driver.find_element(By.XPATH, f"//{tag}[text()=\"{text_new}\"]").text == text:
                                text = text_new
                                break
                        except Exception:
                            logger.info(f"Text check failed: {check}")
                    return add_index_if_needed(f"//{tag}[text()=\"{text}\"]")
                else:
                    # Priority 3: Search by name
                    element_name = element.get_attribute("name")
                    if element_name:
                        return add_index_if_needed(f"//{tag}[@name='{element_name}']")
                    else:
                        # Priority 4: Search by classes
                        element_class = element.get_attribute("class")
                        classes = ".".join(element_class.split())
                        paragraph = False
                        if element_class:
                            return add_index_if_needed(f"//{tag}[contains(@class, '{classes}')]")
                        else:
                            text = element.text.strip()
                            if text and len(text) > 20:
                                paragraph = True
                            # Search in parent elements if the current element does not have the class attribute
                            parent = element.find_element(By.XPATH, "..")
                            while parent:
                                parent_id = parent.get_attribute("id")
                                if parent_id and not is_dynamic_attribute(parent_id):
                                    return add_index_if_needed(f"//{parent.tag_name.lower()}[@id='{parent_id}']",
                                                               paragraph)
                                parent_class = parent.get_attribute("class")
                                if parent_class:
                                    parent_classes = ".".join(parent_class.split())
                                    return add_index_if_needed(
                                        f"//{parent.tag_name.lower()}[contains(@class, '{parent_classes}')]")
                                parent = parent.find_element(By.XPATH, "..")
                            return add_index_if_needed(f"//{tag}[contains(@class, '{classes}')]")

    primary_xpath = generate_primary_xpath()

    # Placeholder locator, if present
    placeholder = element.get_attribute("placeholder")
    placeholder_xpath = f"//{tag}[@placeholder='{placeholder}']" if placeholder else ""

    return primary_xpath + ("\n" + "Placeholder: " + placeholder_xpath if placeholder_xpath else "")


def get_css_selector(driver, element, test_attributes):
    def add_index_if_needed(css_selector):
        siblings = driver.find_elements(By.CSS_SELECTOR, css_selector)
        if len(siblings) > 1:
            index = siblings.index(element) + 1
            return f"{css_selector}:nth-of-type({index})"
        return css_selector

    tag = element.tag_name.lower()

    def is_dynamic_attribute(value):
        # Проверка на динамически сгенерированные атрибуты, например id со случайными числами
        return value.isdigit() or re.match(r".*-\d+", value)

    # Priority 0: Search by test attributes
    primary_css = None
    for attr in test_attributes:
        attr_value = element.get_attribute(attr)
        if attr_value:
            primary_css = f"{tag}[{attr}='{attr_value}']"
            break

    if not primary_css:
        # Priority 1: Search by id
        element_id = element.get_attribute("id")
        if element_id and not is_dynamic_attribute(element_id):
            primary_css = f"{tag}#{element_id}"
        else:
            # Priority 2: Search by name
            element_name = element.get_attribute("name")
            if element_name:
                primary_css = f"{tag}[name='{element_name}']"
            else:
                # Priority 3: Search by classes
                element_class = element.get_attribute("class")
                if element_class:
                    classes = ".".join(element_class.split())
                    primary_css = f"{tag}.{classes}"
                else:
                    # Priority 4: Indexing among tags of the same type
                    primary_css = f"{tag}"

    primary_css = add_index_if_needed(primary_css)

    # Placeholder locator, if present
    placeholder = element.get_attribute("placeholder")
    placeholder_css = f"{tag}[placeholder='{placeholder}']" if placeholder else ""

    # Search in parent elements if the current element does not have the class attribute
    element_class = element.get_attribute("class")
    if not element_class:
        parent = element.find_element(By.XPATH, "..")
        while parent:
            parent_id = parent.get_attribute("id")
            if parent_id and not is_dynamic_attribute(parent_id):
                return add_index_if_needed(f"{parent.tag_name.lower()}#{parent_id}")
            parent_class = parent.get_attribute("class")
            if parent_class:
                parent_classes = ".".join(parent_class.split())
                return add_index_if_needed(f"{parent.tag_name.lower()}.{parent_classes}")
            parent = parent.find_element(By.XPATH, "..")

    return primary_css + ("\n" + "Placeholder: " + placeholder_css if placeholder_css else "")


def copy_to_clipboard(locator):
    """Копирование локатора в буфер обмена."""
    pyperclip.copy(locator)
    print("Локатор скопирован в буфер обмена!")
